
/*
 *  LMT01_Example.ino
 *  Created: July 10th, 2017
 *  This sketch is released AS-IS into the public domain, no guarantee or warranty is given.
 *  This code is not supported by Texas Instruments.
 * 
 *  Description: This sketch shows how to use the LMT01 to read the curent temperature and print
 *  it to a standard 1602 LCD Display. This sketch makes extensive use of the comparator included  
 *  with the Arduino Uno, and as such will not be compatible with other Arduino device. Please see 
 *  the associated schematic file for device setup.
 *
 *  Copyright (C) 2017 Texas Instruments Incorporated - http://www.ti.com/ 
 * 
 *  Redistribution and use in source and binary forms, with or without 
 *  modification, are permitted provided that the following conditions 
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the 
 *    documentation and/or other materials provided with the   
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*/

#include <LiquidCrystal.h>

//Declare RS, E, D4, D5, D6, D7
LiquidCrystal lcd(3,5,6,9,10,11);

volatile int pulseCount = 0;
float temperature = 0;
int hold = 0; 

void setup() 
{
  //initialize 1602 LCD Display
  lcd.begin(16, 2);
  //Print Label for Pulse count
  lcd.setCursor(0,0);
  lcd.write("PULSES: ");
  //Print label for Temperature
  lcd.setCursor(0,1);
  lcd.write("TEMP(C): ");
  
  //Setup ACSR Register, to initlize the comparator

  /*         ACSR Bit Description
  *  
  ACD  - Clear ACD to enable Analog Comparator
  ACBG - Set ACBG to 1 to use internal 1.1V Reference
  ACO  - Clear ACIO (Will be ignored - read only)
  ACI  - Reset Analog Interrupt Flag by writing 1
  ACIE - Set ACIE to enable comparator interrupt
  ACIC - Clear ACIC, no connection to Timer/counter
  ACIS1 - Set ACIS1 to trigger interrupt on falling edge
  ACIS0 - Cleat ACIS0 to trigger interrupt on falling edge
  */
  ACSR = B01011010;    //Set according to above bit description
}

void loop() 
{
  //don't bother entering the loop again if no pulses have been counted yet. 
  if(pulseCount != 0)
  {
    //Wait for counting to be complete
    while(pulseCount != hold)
    {
      hold = pulseCount;
      delay(1);
    }
  
    //Print Pulse count to LCD
    lcd.setCursor(9,0);
    lcd.print(pulseCount);
    
    //Print Temperature to LCD
    temperature = 0.0625 * pulseCount - 50;
    lcd.setCursor(9,1);
    lcd.print(temperature);
  
    
    //reset pulseCount for next loop
    pulseCount = 0;
  }
  delay(2);
}


//Interrupt Service Routine, counts pulses
ISR(ANALOG_COMP_vect) 
{
  //Increment pulse count
  pulseCount += 1;
 
}
